/*
 * This file is part of jSpeed.
 *
 * jSpeed is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * jSpeed is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with jSpeed.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QtGui/QGraphicsTextItem>
#include <QtGui/QGraphicsLineItem>
#include <QtGui/QGraphicsView>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtCore/QString>
#include <QtCore/QDebug>
#include <math.h>
#include "detailscreen.h"
#include "odometer.h"
#include "graphicsscene.h"

namespace
{
    QString const FONT = "Tahoma";
    QString const BACKGROUND = ":/resources/themes/default/background.png";
    int const FONT_SIZE = 20;
    int const PADDING_LEFT = 40;
    int const START_HEIGHT = 80;
    int const ITEM_WIDTH = 130;
}

DetailScreen::DetailScreen(QWidget* parent): GraphicsScreen(parent)
{
    QString unit = Odometer::getUnit();
    QString speedUnit = Odometer::getSpeedUnit();

    QGraphicsPixmapItem* background = new QGraphicsPixmapItem(QPixmap(BACKGROUND));
    getScene()->addItem(background);

    tripLabel_ = createItem(tr("Trip"));
    trip_ = createItem(roundDouble(Odometer::instance().getTrip()));
    tripUnit_ = createItem(unit);

    totalLabel_ = createItem(tr("Total"));
    total_ = createItem(roundDouble(Odometer::instance().getTotal()));
    totalUnit_ = createItem(unit);

    speedLabel_ = createItem(tr("Speed"));
    speed_ = createItem(roundDouble(0.0));
    speedUnit_ = createItem(speedUnit);

    avgSpeedLabel_ = createItem(tr("Average speed"));
    avgSpeed_ = createItem(roundDouble(Odometer::instance().getAverageSpeed()));
    avgSpeedUnit_ = createItem(speedUnit);

    maxSpeedLabel_ = createItem(tr("Max speed"));
    maxSpeed_ = createItem(roundDouble(Odometer::instance().getMaxSpeed()));
    maxSpeedUnit_ = createItem(speedUnit);

    tripTimeLabel_ = createItem(tr("Trip time"));
    tripTime_ = createItem(formatTime(Odometer::instance().getTripTime()));

    totalTimeLabel_ = createItem(tr("Total time"));
    totalTime_ = createItem(formatTime(Odometer::instance().getTotalTime()));

    line1_ = createLine();
    line2_ = createLine();

    connect(&(Odometer::instance()), SIGNAL(timeUpdated()), this, SLOT(updateTime()));
    connect(&(Odometer::instance()), SIGNAL(unitChanged()), this, SLOT(updateUnits()));

    reArrange();
}

void DetailScreen::update()
{
    speed_->setPlainText(roundDouble(Odometer::instance().getLatestFix().speed));
    trip_->setPlainText(roundDouble(Odometer::instance().getTrip()));
    total_->setPlainText(roundDouble(Odometer::instance().getTotal()));
    avgSpeed_->setPlainText(roundDouble(Odometer::instance().getAverageSpeed()));
    maxSpeed_->setPlainText(roundDouble(Odometer::instance().getMaxSpeed()));
}

void DetailScreen::updateTime()
{
    tripTime_->setPlainText(formatTime(Odometer::instance().getTripTime()));
    totalTime_->setPlainText(formatTime(Odometer::instance().getTotalTime()));
}

void DetailScreen::updateUnits()
{
    QString unit = Odometer::getUnit();
    QString speedUnit = Odometer::getSpeedUnit();

    tripUnit_->setPlainText(unit);
    totalUnit_->setPlainText(unit);
    speedUnit_->setPlainText(speedUnit);
    avgSpeedUnit_->setPlainText(speedUnit);
    maxSpeedUnit_->setPlainText(speedUnit);

    update();
}

void DetailScreen::reArrange()
{
    GraphicsScreen::reArrange();

    int lineHeight = getScene()->height() / 9;

    int area1 = (getScene()->width() / 3) + ITEM_WIDTH / 2 + 20;
    int area2 = area1 + ITEM_WIDTH;

    tripLabel_->setPos(PADDING_LEFT, START_HEIGHT);
    trip_->setPos(area1, START_HEIGHT);
    tripUnit_->setPos(area2, START_HEIGHT);

    totalLabel_->setPos(PADDING_LEFT, START_HEIGHT + lineHeight);
    total_->setPos(area1, START_HEIGHT + lineHeight);
    totalUnit_->setPos(area2, START_HEIGHT + lineHeight);

    speedLabel_->setPos(PADDING_LEFT, START_HEIGHT + 2 * lineHeight);
    speed_->setPos(area1, START_HEIGHT + 2 * lineHeight);
    speedUnit_->setPos(area2, START_HEIGHT + 2 * lineHeight);

    avgSpeedLabel_->setPos(PADDING_LEFT, START_HEIGHT + 3 * lineHeight);
    avgSpeed_->setPos(area1, START_HEIGHT + 3 * lineHeight);
    avgSpeedUnit_->setPos(area2, START_HEIGHT + 3 * lineHeight);

    maxSpeedLabel_->setPos(PADDING_LEFT, START_HEIGHT + 4 * lineHeight);
    maxSpeed_->setPos(area1, START_HEIGHT + 4 * lineHeight);
    maxSpeedUnit_->setPos(area2, START_HEIGHT + 4 * lineHeight);

    tripTimeLabel_->setPos(PADDING_LEFT, START_HEIGHT + 5 * lineHeight);
    tripTime_->setPos(area1, START_HEIGHT + 5 * lineHeight);

    totalTimeLabel_->setPos(PADDING_LEFT, START_HEIGHT + 6 * lineHeight);
    totalTime_->setPos(area1, START_HEIGHT + 6 * lineHeight);

    int y1 = START_HEIGHT + 2 * lineHeight - lineHeight / 2 + FONT_SIZE + 2;
    int y2 = START_HEIGHT + 5 * lineHeight - lineHeight / 2 + FONT_SIZE + 2;
    int x = getScene()->width() - PADDING_LEFT;
    line1_->setLine(PADDING_LEFT + 2, y1, x, y1);
    line2_->setLine(PADDING_LEFT + 2, y2, x, y2);
}

QGraphicsTextItem* DetailScreen::createItem(QString const& text)
{
    QGraphicsTextItem* item = new QGraphicsTextItem(text);
    QFont font(FONT, FONT_SIZE);
    font.setBold(true);
    item->setFont(font);
    item->setDefaultTextColor(QColor("#fff"));
    getScene()->addItem(item);
    return item;
}

QGraphicsLineItem* DetailScreen::createLine()
{
    QGraphicsLineItem* item = new QGraphicsLineItem;
    QPen pen(QColor("#ddd"));
    pen.setWidth(1);
    item->setPen(pen);
    getScene()->addItem(item);
    return item;
}

QString DetailScreen::roundDouble(double number)
{
    QString result;
    result.sprintf("%.2lf", number);
    return result;
}

QString DetailScreen::formatTime(qulonglong time)
{
    qulonglong secs = time % 60;
    qulonglong mins_tmp = time / 60;
    qulonglong mins = mins_tmp % 60;
    qulonglong hours = mins_tmp / 60;

    static char format[] = "%02d";

    return QString().sprintf(format, hours) + ":" +
           QString().sprintf(format, mins) + ":" +
           QString().sprintf(format, secs);
}
